/*	Batch Ask for MS-DOS
	by Ivan L. Baggett

	Batch Ask v1.1 (C) 2000 Bagotronix

	Command line parameters:
	n       Timeout value in ticks (n = 1 to 32767)

	There are 18.2 ticks per second.  Example:  'batchask 37' will wait up to 2
	seconds for the 'Y' key, then continue with an exit code of 0 if no 'Y' key
	was pressed.  If the 'Y' key was pressed before 2 seconds have elapsed, it
	will continue with an exit code of 1.  If a key other than 'Y' is pressed
	it will continue immediately with an exit code of 0.
*/

#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <stdlib.h>

#define BIOS_TIMER 0x1c		// BIOS timer tick hook

int ticker;
void interrupt (*oldtimervect)(void);

/*	timer_ISR():  hooks BIOS INT 1Ch timer service, which happens 18.2 times
	per second
*/
void interrupt timer_ISR (void) {
	++ticker;
	oldtimervect();		// call old vector also
}


/*	restore INT 1Ch back to original vector
*/
void restore_ISR (void) {
	disable();
	setvect(BIOS_TIMER, oldtimervect);
	enable();
}


int main (int argc, char *argv[]) {
	int idelay;				/* delay in seconds */
	int c, i;


	ticker = 0;

	/* initialize the IRQ: set vector */
	disable();
	oldtimervect = getvect(BIOS_TIMER);
	setvect(BIOS_TIMER, timer_ISR);
	enable();

	idelay = 0;
	if (argc >= 2) {
		idelay = atoi (argv[1]);
	}
	if (argc < 2 || idelay <= 0) {
		restore_ISR();
		return (1);				// error - invalid argument
	}
	printf ("\nTerminate batch operation? (y/N)");
	printf ("\nTicks remaining:      ");

	while (ticker < idelay + 1) {
		i = ticker;
		printf ("\x8\x8\x8\x8\x8");	// backspaces
		printf ("%5d", idelay - ticker);
		if (kbhit ()) {
			c = getch ();
			if (!c)
				getch ();	/* if func key, grab 2nd code */
			if (c == 'Y' || c == 'y') {
				restore_ISR();
				return (1);		/* terminate batch op */
			}
			else {
				restore_ISR();
				return (0);		/* continue batch op */
			}
		}
		while (i == ticker);	// wait until next tick to loop again
	}

	restore_ISR();
	return (0);		/* timed out */
}
